@@ -5,6 +5,11 @@ module Servant.Client.Core.Reexport
   (
     -- * HasClient
     HasClient(..)
+  , foldMapUnion
+  , matchUnion
+  , AsClientT
+  , (//)
+  , (/:)
 
     -- * Response (for @Raw@)
   , Response
@@ -21,6 +26,7 @@ module Servant.Client.Core.Reexport
   , showBaseUrl
   , parseBaseUrl
   , InvalidBaseUrlException
+
   ) where
 
 
